# -*- coding: utf-8 -*-
"""Models"""

from .customvocabulary import CustomVocabulary
from .streaming import MediaConfig
from .asynchronous import Job, JobStatus, Account, Transcript, Monologue, Element, CaptionType



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev.ai"""

from .baseclient import BaseClient
from . import utils

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin


class RevAiCustomVocabulariesClient(BaseClient):
    """Client which implements Rev.ai CustomVocabulary API
    See https://www.rev.ai/docs/streaming#section/WebSocket-Endpoint/Custom-Vocabulary
    """

    def __init__(self, access_token):
        """Constructor

        :param access_token: access token which authorizes all requests and
                             links them to your account. Generated on the
                             settings page of your account dashboard
                             on Rev.ai
        """
        BaseClient.__init__(self, access_token)

        self.base_url = urljoin(self.base_url, 'vocabularies/')

    def submit_custom_vocabularies(
            self,
            custom_vocabularies,
            callback_url=None,
            metadata=None):
        """Submit custom vocabularies.
        See https://www.rev.ai/docs/streaming#operation/SubmitCustomVocabulary

        :param custom_vocabularies: List of CustomVocabulary objects
        :param callback_url: callback url to invoke on job completion as a
                             webhook
        :param metadata: info to associate with the transcription job
        """

        if not custom_vocabularies:
            raise ValueError('custom_vocabularies must be provided')

        payload = self._create_custom_vocabularies_options_payload(
            custom_vocabularies,
            callback_url,
            metadata
        )

        response = self._make_http_request(
            "POST",
            self.base_url,
            json=payload
        )

        return response.json()

    def get_custom_vocabularies_information(self, id):
        """ Get the custom vocabulary status
        See https://www.rev.ai/docs/streaming#operation/GetCustomVocabulary

        :param id: string id of custom vocabulary submission
        """

        response = self._make_http_request("GET", urljoin(self.base_url, id))
        return response.json()

    def get_list_of_custom_vocabularies(self, limit=None):
        """ Get a list of custom vocabularies
        See https://www.rev.ai/docs/streaming#operation/GetCustomVocabularies

        :param limit: optional, limits the number of jobs returned
        """

        url = self.base_url
        if limit:
            url += '?limit={}'.format(limit)

        response = self._make_http_request("GET", url)
        return response.json()

    def delete_custom_vocabulary(self, id):
        """ Delete a custom vocabulary
        See https://www.rev.ai/docs/streaming#operation/DeleteCustomVocabulary

        :param id: string id of custom vocabulary to be deleted
        :returns: None if job was successfully deleted
        :raises: HTTPError
        """

        self._make_http_request("DELETE", urljoin(self.base_url, id))
        return

    def _create_custom_vocabularies_options_payload(
            self,
            custom_vocabularies,
            callback_url=None,
            metadata=None):
        payload = {}
        if callback_url:
            payload['callback_url'] = callback_url
        if metadata:
            payload['metadata'] = metadata
        if custom_vocabularies:
            payload['custom_vocabularies'] =\
                utils._process_vocabularies(custom_vocabularies)
        return payload



# -*- coding: utf-8 -*-
"""Job model"""

from .job_status import JobStatus


class Job:
    def __init__(
            self, id_, created_on, status,
            completed_on=None,
            name=None,
            callback_url=None,
            metadata=None,
            media_url=None,
            failure=None,
            failure_detail=None,
            duration_seconds=None):
        """
        :param id_: unique id of job
        :param created_on: date and time at which this job was started
        :param status: current job status 'IN_PROGRESS', 'TRANSCRIBED',
                       or 'FAILED'
        :param completed_on: date and time at which this job finished
                             being transcribed
        :param name: name of submitted file if local file was used
        :param callback_url: callback_url if provided
        :param metadata: metadata if provided
        :param media_url: url of transcribed media if job was submitted
                          this way
        :param failure: type of failure if job has failed
        :param failure_detail: more detailed failure message if job has failed
        :param duration_seconds: duration of submitted file in seconds
        """
        self.id = id_
        self.created_on = created_on
        self.status = status
        self.completed_on = completed_on
        self.name = name
        self.callback_url = callback_url,
        self.metadata = metadata
        self.media_url = media_url
        self.failure = failure
        self.failure_detail = failure_detail
        self.duration_seconds = duration_seconds

    def __eq__(self, other):
        """Override default equality operator"""
        if isinstance(other, self.__class__):
            return self.__dict__ == other.__dict__
        return False

    @classmethod
    def from_json(cls, json):
        """Alternate constructor used for parsing json"""
        return cls(
            json['id'],
            json['created_on'],
            JobStatus.from_string(json['status']),
            json.get('completed_on'),
            json.get('name'),
            json.get('callback_url'),
            json.get('metadata'),
            json.get('media_url'),
            json.get('failure'),
            json.get('failure_detail'),
            json.get('duration_seconds')
        )



# -*- coding: utf-8 -*-
"""Unit tests for account endpoints"""

import pytest
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models.asynchronous import Account

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

URL = urljoin(RevAiAPIClient.base_url, 'account')


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestAccountEndpoints():
    def test_get_account_with_success(
            self, mock_session, make_mock_response):
        email = 'text@example.com'
        seconds = 10
        data = {'email': email, 'balance_seconds': seconds}
        response = make_mock_response(url=URL, json_data=data)
        mock_session.request.return_value = response
        client = RevAiAPIClient("token")

        res = client.get_account()

        assert res == Account(email, seconds)
        mock_session.request.assert_called_once_with("GET", URL, headers=client.default_headers)



# -*- coding: utf-8 -*-
"""Unit tests for the media config class"""

from src.rev_ai.models.streaming import MediaConfig


class TestMediaConfig():
    def test_constructor(self):
        example_config = MediaConfig('content_type',
                                     'layout',
                                     'rate',
                                     'format',
                                     'channels')

        assert example_config.content_type == 'content_type'
        assert example_config.layout == 'layout'
        assert example_config.rate == 'rate'
        assert example_config.format == 'format'
        assert example_config.channels == 'channels'

    def test_constructor_with_defaults(self):
        example_config = MediaConfig()

        assert example_config.content_type == 'audio/*'
        assert not example_config.layout
        assert not example_config.rate
        assert not example_config.format
        assert not example_config.channels

    def test_get_content_type_string_all(self):
        example_config = MediaConfig('content_type',
                                     'LAYOUT',
                                     'RATE',
                                     'FORMAT',
                                     'CHANNELS')

        content_type_string = example_config.get_content_type_string()

        assert content_type_string == ('content_type'
                                       ';layout=LAYOUT'
                                       ';rate=RATE'
                                       ';format=FORMAT'
                                       ';channels=CHANNELS')

    def test_get_content_type_string_missing(self):
        example_config = MediaConfig('content_type',
                                     '',
                                     '',
                                     '',
                                     'CHANNELS')

        content_type_string = example_config.get_content_type_string()

        assert content_type_string == 'content_type;channels=CHANNELS'



# -*- coding: utf-8 -*-

import pytest
from src.rev_ai.apiclient import RevAiAPIClient
from src.rev_ai.models import CaptionType

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
TOKEN = "token"
URL = urljoin(RevAiAPIClient.base_url, 'jobs/{}/captions'.format(JOB_ID))


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestCaptionEndpoint():
    def test_get_captions_default_content_type(self, mock_session, make_mock_response):
        data = 'Test'
        expected_content_type = CaptionType.SRT.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_captions(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            URL,
            headers=expected_headers
        )

    @pytest.mark.parametrize('content_type', [CaptionType.SRT, CaptionType.VTT])
    def test_get_captions_with_content_type(self, content_type, mock_session, make_mock_response):
        data = 'Test'
        expected_content_type = content_type.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_captions(JOB_ID, content_type)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            URL,
            headers=expected_headers
        )

    def test_get_captions_with_speaker_channel(self, mock_session, make_mock_response):
        data = 'Test'
        channel_id = 1
        expected_url = URL + '?speaker_channel={}'.format(channel_id)
        expected_content_type = CaptionType.SRT.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=expected_url, text=data)
        mock_session.request.return_value = response

        res = client.get_captions(JOB_ID, channel_id=channel_id)

        assert res == data
        mock_session.request.assert_called_once_with(
            "GET",
            expected_url,
            headers=expected_headers
        )

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_captions_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_captions(id)

    def test_get_captions_as_stream_default_content_type(self, mock_session, make_mock_response):
        data = 'Test'
        expected_content_type = CaptionType.SRT.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_captions_as_stream(JOB_ID)

        assert res.content == data
        mock_session.request.assert_called_once_with(
            "GET",
            URL,
            headers=expected_headers,
            stream=True
        )

    @pytest.mark.parametrize('content_type', [CaptionType.SRT, CaptionType.VTT])
    def test_get_captions_as_stream_with_content_type(self, content_type,
                                                      mock_session, make_mock_response):
        data = 'Test'
        expected_content_type = content_type.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_captions_as_stream(JOB_ID, content_type)
        assert res.content == data
        mock_session.request.assert_called_once_with(
            "GET",
            URL,
            headers=expected_headers,
            stream=True
        )

    def test_get_captions_as_stream_with_speaker_channel(self, mock_session, make_mock_response):
        data = 'Test'
        channel_id = 8
        expected_url = URL + '?speaker_channel={}'.format(channel_id)
        expected_content_type = CaptionType.SRT.value
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': expected_content_type}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=expected_url, text=data)
        mock_session.request.return_value = response

        res = client.get_captions_as_stream(JOB_ID, channel_id=channel_id)
        assert res.content == data
        mock_session.request.assert_called_once_with(
            "GET",
            expected_url,
            headers=expected_headers,
            stream=True
        )

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_captions_as_stream_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_captions_as_stream(id)



# -*- coding: utf-8 -*-
"""Async Models"""

from .caption_type import CaptionType
from .job import Job
from .job_status import JobStatus
from .account import Account
from .transcript import Transcript, Monologue, Element



# -*- coding: utf-8 -*-
"""Unit tests for transcript endpoints"""

import pytest
import json
from src.rev_ai.models.asynchronous import Transcript, Monologue, Element
from src.rev_ai.apiclient import RevAiAPIClient

try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

JOB_ID = '1'
TOKEN = "token"
URL = urljoin(RevAiAPIClient.base_url, 'jobs/{}/transcript'.format(JOB_ID))


@pytest.mark.usefixtures('mock_session', 'make_mock_response')
class TestTranscriptEndpoints():
    def test_get_transcript_text(self, mock_session, make_mock_response):
        data = 'Test'
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': 'text/plain'}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_transcript_text(JOB_ID)

        assert res == data
        mock_session.request.assert_called_once_with("GET",
                                                     URL,
                                                     headers=expected_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_text_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_transcript_text(id)

    def test_get_transcript_text_as_stream(self, mock_session, make_mock_response):
        data = 'Test'
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': 'text/plain'}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, text=data)
        mock_session.request.return_value = response

        res = client.get_transcript_text_as_stream(JOB_ID)

        assert res.content == data
        mock_session.request.assert_called_once_with("GET",
                                                     URL,
                                                     headers=expected_headers,
                                                     stream=True)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_text_as_stream_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_transcript_text_as_stream(id)

    def test_get_transcript_json(self, mock_session, make_mock_response):
        data = {
            'monologues': [{
                'speaker': 1,
                'elements': [{
                    'type': 'text',
                    'value': 'Hello',
                    'ts': 0.75,
                    'end_ts': 1.25,
                    'confidence': 0.85
                }]
            }]
        }
        expected = json.loads(json.dumps(data))
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, json_data=data)
        mock_session.request.return_value = response

        res = client.get_transcript_json(JOB_ID)

        assert res == expected
        mock_session.request.assert_called_once_with(
            "GET", URL, headers=expected_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_json_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_transcript_json(id)

    def test_get_transcript_json_as_stream(self, mock_session, make_mock_response):
        data = {
            'monologues': [{
                'speaker': 1,
                'elements': [{
                    'type': 'text',
                    'value': 'Hello',
                    'ts': 0.75,
                    'end_ts': 1.25,
                    'confidence': 0.85
                }]
            }]
        }
        expected = json.loads(json.dumps(data))
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, json_data=data)
        mock_session.request.return_value = response

        res = client.get_transcript_json_as_stream(JOB_ID)

        assert json.loads(res.content.decode('utf-8').replace("\'", "\"")) == expected
        mock_session.request.assert_called_once_with(
            "GET", URL, headers=expected_headers, stream=True)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_json_as_stream_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_transcript_json_as_stream(id)

    def test_get_transcript_object_with_success(self, mock_session, make_mock_response):
        data = {
            'monologues': [{
                'speaker': 1,
                'elements': [{
                    'type': 'text',
                    'value': 'Hello',
                    'ts': 0.75,
                    'end_ts': 1.25,
                    'confidence': 0.85
                }]
            }]
        }
        expected = Transcript([Monologue(1, [Element('text', 'Hello', 0.75, 1.25, 0.85)])])
        client = RevAiAPIClient(TOKEN)
        expected_headers = {'Accept': 'application/vnd.rev.transcript.v1.0+json'}
        expected_headers.update(client.default_headers)
        response = make_mock_response(url=URL, json_data=data)
        mock_session.request.return_value = response

        res = client.get_transcript_object(JOB_ID)

        assert res == expected
        mock_session.request.assert_called_once_with(
            "GET", URL, headers=expected_headers)

    @pytest.mark.parametrize('id', [None, ''])
    def test_get_transcript_object_with_no_job_id(self, id, mock_session):
        with pytest.raises(ValueError, match='id_ must be provided'):
            RevAiAPIClient(TOKEN).get_transcript_object(id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from glob import glob
from setuptools import setup, find_packages
import re
import ast
import os


def parse_requirements(filehandle):
    for line in filehandle:
        line = line.strip()
        if line == '':
            continue
        # Comments are lines that start with # only.
        elif not line or line.startswith('#'):
            continue
        elif (line.startswith('-r') or line.startswith('--requirement') or
              line.startswith('-f') or line.startswith('--find-links') or
              line.startswith('-i') or line.startswith('--index-url') or
              line.startswith('--extra-index-url') or line.startswith('--no-index') or
              line.startswith('-Z') or line.startswith('--always-unzip')):
            continue
        else:
            yield line


_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('src/rev_ai/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt', 'r') as req_file:
    requirements = list(parse_requirements(req_file))

with open('requirements_dev.txt', 'r') as req_dev_file:
    test_requirements = list(parse_requirements(req_dev_file))

setup(
    name='rev_ai',
    version=version,
    description='Rev.ai makes speech applications easy to build!',
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    author="Rev Ai",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[os.path.splitext(os.path.basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    license='MIT license',
    keywords='rev_ai',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    setup_requires=['pytest-runner==4.2'],
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/revdotcom/revai-python-sdk',
)



# -*- coding: utf-8 -*-
"""Speech recognition tools for using Rev.ai"""

from . import CustomVocabulary


def _process_vocabularies(unprocessed_vocabularies):
    """
    This method takes in a list that contains CustomVocabulary objects
    and returns a list in which any such objects are converted properly
    to custom vocabulary dictionaries. Any items of other types in the list
    are not changed.
    """
    return list(map(lambda custom_vocabulary: custom_vocabulary.to_dict()
                    if isinstance(custom_vocabulary, CustomVocabulary)
                    else custom_vocabulary, unprocessed_vocabularies))



class CustomVocabulary:
    """CustomVocabulary object to provide a clean way
    to create custom vocabularies. CustomVocabulary objects
    can be used in the various api clients."""

    def __init__(self, phrases):
        """Constructor

        :param phrases: list of strings of custom phrases to be recognized in
                        submitted audio
        """
        self.phrases = [phrase for phrase in phrases]

    def to_dict(self):
        """Returns the raw form of the custom vocabularies as the api
        expects them"""
        return {'phrases': self.phrases}
